#!/usr/bin/env python3

from time import sleep
from subprocess import run, DEVNULL, PIPE

from pingparsing import PingParsing
from pingparsing import PingTransmitter

from typing import Any, Dict, Optional

DEFAULT_TARGET = "httpbin.com"
DEFAULT_SLEEP_BETWEEN = 10
DEFAULT_TEST_COUNT = 2


def print_verbose(string: str, verbose: bool = True):
    """
    Verbosity print.

    :param string: str: String to print.
    :param verbose: bool: Enable or disable verbosity.
        (Default value = True)

    """
    if verbose:
        print(string)


def test_connection_by_ping(
    page: str = DEFAULT_TARGET,
    count: int = DEFAULT_TEST_COUNT,
        ) -> Dict[Any, Any]:
    """
    Ping the desired host and return the results.
    :param page: str: Page to target on the test.
        (Default value = DEFAULT_TARGET)
    :param count: int: Number of attemps on the test.
        (Default value = DEFAULT_TEST_COUNT)

    """
    ping_parser = PingParsing()
    transmitter = PingTransmitter()
    transmitter.destination = page
    transmitter.count = 2
    result = transmitter.ping()

    return ping_parser.parse(result).as_dict()


def connect_to_wifi(essid: str, password: Optional[str]) -> bool:
    """
    It tries to connect to an AP using nmcli.

    :param essid: str: AP's ESSID.
    :param password: Optional[str]: AP's password.

    """
    if essid and password:
        completed_process = run(
            f'nmcli device wifi connect "{essid}" password "{password}"',
            shell=True,
            stdout=PIPE,
            stderr=DEVNULL
            )
        if completed_process.stdout == b"":
            result = True
        else:
            result = False
    elif essid:
        completed_process = run(
            f'nmcli conn up "{essid}"',
            shell=True,
            stdout=PIPE,
            stderr=DEVNULL
            )
        if b"successfully" in completed_process.stdout:
            result = True
        else:
            result = False
    else:
        result = False
    return result


def loop_test_and_reconnect(
    essid: str,
    password: Optional[str] = None,
    sleep_between: int = DEFAULT_SLEEP_BETWEEN,
    test_target: str = DEFAULT_TARGET,
    verbose: bool = False,
    test_count: int = DEFAULT_TEST_COUNT
        ):
    """
    Run a connectivity test and try to reconnect if test failed.

    :param essid: str: AP's ESSID.
    :param password: Optional[str]: AP's password (Default value = None)
    :param sleep_between: int: Time between connectivity tests.
        (Default value = DEFAULT_SLEEP_BETWEEN)
    :param test_target: str: Page used to do the connectivity test.
        (Default value = DEFAULT_TARGET)
    :param verbose: bool: Verbosity (Default value = False)
    :param test_count: int: Number of attemps on the test.
        (Default value = DEFAULT_TEST_COUNT)

    See your networks with "nmcli conn" (on the shell).
    """
    while True:
        try:
            connection_test_result = test_connection_by_ping(
                page=test_target,
                count=test_count
                )
            connected = True
        except Exception:
            print_verbose(string="·! Connection test failed.", verbose=verbose)
            connected = False

        if connected:
            print_verbose(
                string="· Connection test ran successfully.", verbose=verbose)
        else:
            print_verbose(string="- Trying to connect.", verbose=verbose)
            connected = connect_to_wifi(
                essid=essid,
                password=password
                )
        sleep(sleep_between)



#!/usr/bin/env python3

from setuptools import setup
from pathlib import Path

readme = Path("README.md").read_text()


setup(
    name="wifireconnect",
    packages=["wifireconnect"],
    entry_points={
        "console_scripts": [
            "wifireconnect = wifireconnect.__main__:main"
        ]
    },
    version="0.1.0",
    license="GPL3",
    description="Python3 module to test and ensure connectivity "
    "on a network which have stability problems on"
    " traffic routing (link layer).",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Carlos A. Planchón",
    author_email="bubbledoloresuruguay2@gmail.com",
    url="https://github.com/carlosplanchon/wifireconnect",
    download_url="https://github.com/carlosplanchon/"
        "wifireconnect/archive/v0.1.0.tar.gz",
    keywords=["wifi", "connectivity", "networking"],
    install_requires=[
        "pingparsing"
    ],
    classifiers=[
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3.7",
    ],
)



#!/usr/bin/env python3

from wifireconnect.wifireconnect import print_verbose
from wifireconnect.wifireconnect import test_connection_by_ping
from wifireconnect.wifireconnect import connect_to_wifi
from wifireconnect.wifireconnect import loop_test_and_reconnect



#!/usr/bin/env python3

from wifireconnect import loop_test_and_reconnect
from wifireconnect import DEFAULT_TARGET, DEFAULT_SLEEP_BETWEEN
from wifireconnect import DEFAULT_SLEEP_BETWEEN
from wifireconnect import DEFAULT_TEST_COUNT

from argparse import ArgumentParser


def main():
    parser = ArgumentParser(
        description="Run a connectivity test on a network connection"
        "(using ping) every a 's' time and try to reconnect if that test fail.\n"
        "(See your networks with 'nmcli conn')"
        )
    # Argparse logic.
    parser.add_argument(
            "-v", "--verbose",
            help="Verbosity.",
            action="store_true",
            dest="verbose",
            default=False
        )
    parser.add_argument(
            "-t", "--target",
            type=str,
            help=f"Target webpage. Default {DEFAULT_TARGET}.",
            dest="target",
            default=DEFAULT_TARGET
        )
    parser.add_argument(
            "-s", "--sleep",
            type=int,
            help="Time between connectivity tests."
            f" Default: {DEFAULT_SLEEP_BETWEEN}",
            dest="sleep_between",
            default=DEFAULT_SLEEP_BETWEEN
        )
    parser.add_argument(
            "-c", "--count",
            type=int,
            help="Num of attemps to ping target on the connectivity test."
            f" Default: {DEFAULT_TEST_COUNT}",
            dest="count",
            default=DEFAULT_TEST_COUNT
        )
    parser.add_argument(
            "-e", "--essid",
            type=str,
            help="AP's ESSID.",
            dest="essid",
        )
    parser.add_argument(
            "-p", "--password",
            type=str,
            help="AP's password.",
            dest="password",
            default=None
        )
    args = parser.parse_args()

    loop_test_and_reconnect(
        essid=args.essid,
        password=args.password,
        sleep_between=args.sleep_between,
        test_target=args.target,
        verbose=args.verbose,
        test_count=args.count
            )
